@@ -35,6 +35,7 @@
 			rc;})
 
 #define MIN(x,y) ((x < y) ? x : y)
+#define MAX(a, b) ((a) > (b) ? (a) : (b))
 
 #define DEBUGFS_POWERPC "/sys/kernel/debug/powerpc"
 #define DEBUGFS_SCOM DEBUGFS_POWERPC"/scom"
@@ -55,6 +56,17 @@
 			  HID0_SINGLE_DECODE | HID0_EN_INST_TRACE |\
 			  HID0_TRACE_EN)
 
+#define HIDQ_REGISTER			0x20028415
+#define  HIDQ_ONE_PPC			PPC_BIT(0)
+#define  HIDQ_EN_INST_TRACE		PPC_BIT(1)
+#define  HIDQ_HILE			PPC_BIT(4)
+#define  HIDQ_DIS_PROC_REC		PPC_BIT(5)
+#define  HIDQ_TRACE_BITS (HIDQ_ONE_PPC | HIDQ_EN_INST_TRACE)
+
+#define CORE_FIR			0x20028440
+#define  CORE_FIR_PC_LOGIC		PPC_BIT(45)
+#define CORE_FIR_AND			0x20028441
+#define CORE_FIR_MASK			0x20028443
 /*
  * This is a CORE register not a HTM register, don't pass the HTM
  * target to it.
@@ -62,24 +74,58 @@
 #define NCU_MODE_REGISTER		0x10C0A
 #define NCU_MODE_HTM_ENABLE		PPC_BIT(0)
 
+#define IMA_EVENT_MASK			0x20020400
+
 #define HTM_COLLECTION_MODE		0
 #define	  HTM_MODE_ENABLE		PPC_BIT(0)
 #define	  HTM_MODE_CONTENT_SEL		PPC_BITMASK(1,2)
+#define   HTM_MODE_CORE_TRACE_EN	PPC_BIT(1)
+#define   HTM_MODE_LLAT_TRACE_EN	PPC_BIT(2)
+#define   HTM_MODE_DMW_TRACE_EN		PPC_BIT(3)
 #define   NHTM_MODE_CAPTURE		PPC_BITMASK(4,12)
 #define     NHTM_MODE_CAPTURE_PMISC	PPC_BIT(5)
 #define     NHTM_MODE_CRESP_PRECISE	PPC_BIT(6)
 #define     CHTM_MODE_NO_ASSERT_LLAT_L3 PPC_BIT(6)
 #define	  HTM_MODE_WRAP			PPC_BIT(13)
+#define	  HTM_MODE_DIS_TIMESTAMP	PPC_BIT(14)
+#define	  HTM_MODE_FULL_ON_ERR		PPC_BIT(8)
+#define	  HTM_MODE_DISABLE_STALL	PPC_BIT(16)
 #define HTM_MEMORY_CONF			1
 #define   HTM_MEM_ALLOC			PPC_BIT(0)
 #define   HTM_MEM_SCOPE			PPC_BITMASK(1,3)
 /* Note: the next 3 change on P9 */
 #define   HTM_MEM_PRIORITY		PPC_BITMASK(4,5)
+#define   HTM_MEM_PRIORITY_P10		PPC_BITMASK(4)
 #define   HTM_MEM_SIZE_SMALL		PPC_BIT(13)
+#define   HTM_MEM_SIZE_SMALL_P10	PPC_BIT(5)
 #define   HTM_MEM_BASE			PPC_BITMASK(14,39)
+#define   HTM_MEM_BASE_P10		PPC_BITMASK(8,39)
 #define	  HTM_MEM_SIZE			PPC_BITMASK(40,48)
 #define HTM_STATUS			2
+#define   HTM2_STATUS_MASK		PPC_BITMASK(22,39)
+#define   HTM2_STATUS_CRESP_OV		PPC_BIT(22)
+#define	  HTM2_STATUS_REPAIR		PPC_BIT(23)
+#define   HTM2_STATUS_BUF_WAIT		PPC_BIT(24)
+#define   HTM2_STATUS_TRIG_DROPPED	PPC_BIT(25)
+#define   HTM2_STATUS_ADDR_ERROR	PPC_BIT(26)
+#define   HTM2_STATUS_REC_DROPPED	PPC_BIT(27)
+#define   HTM2_STATUS_INIT		PPC_BIT(28)
+#define   HTM2_STATUS_PREREQ		PPC_BIT(29)
+#define   HTM2_STATUS_READY		PPC_BIT(30)
+#define   HTM2_STATUS_TRACING		PPC_BIT(31)
+#define   HTM2_STATUS_PAUSED		PPC_BIT(32)
+#define   HTM2_STATUS_FLUSH		PPC_BIT(33)
+#define   HTM2_STATUS_COMPLETE		PPC_BIT(34)
+#define   HTM2_STATUS_ENABLE		PPC_BIT(35)
+#define   HTM2_STATUS_STAMP		PPC_BIT(36)
+#define   HTM2_STATUS_SCOM_ERROR	PPC_BIT(37)
+#define   HTM2_STATUS_PARITY_ERROR	PPC_BIT(38)
+#define   HTM2_STATUS_INVALID_CRESP	PPC_BIT(39)
+#define   HTM2_STATUS_STATE_MASK	(PPC_BITMASK(28,36) | HTM_STATUS_REPAIR)
+#define	  HTM2_STATUS_ERROR_MASK	(HTM_STATUS_CRESP_OV | PPC_BITMASK(24,27) | PPC_BITMASK(37,39))
 #define   HTM_STATUS_MASK		PPC_BITMASK(2,19)
+#define   HTM_STATUS_PURGING		PPC_BIT(0)
+#define   HTM_STATUS_PURGED		PPC_BIT(1)
 #define   HTM_STATUS_CRESP_OV		PPC_BIT(2)
 #define	  HTM_STATUS_REPAIR		PPC_BIT(3)
 #define   HTM_STATUS_BUF_WAIT		PPC_BIT(4)
@@ -102,7 +148,9 @@
 #define	  HTM_STATUS_ERROR_MASK		(HTM_STATUS_CRESP_OV | PPC_BITMASK(4,7) | PPC_BITMASK(17,19))
 #define HTM_LAST_ADDRESS		3
 #define   HTM_LAST_ADDRESS_MASK		PPC_BITMASK(8,56)
-#define HTM_SCOM_TRIGGER		4
+#define HTM2_LAST_ADDRESS		4
+#define   HTM2_LAST_ADDRESS_MASK	PPC_BITMASK(8,56)
+#define HTM_SCOM_TRIGGER		(4 + dual_htm_offset)
 #define	  HTM_TRIG_START		PPC_BIT(0)
 #define   HTM_TRIG_STOP			PPC_BIT(1)
 #define   HTM_TRIG_PAUSE		PPC_BIT(2)
@@ -110,28 +158,57 @@
 #define   HTM_TRIG_RESET		PPC_BIT(4)
 #define   HTM_TRIG_MARK_VALID		PPC_BIT(5)
 #define   HTM_TRIG_MARK_TYPE		PPC_BITMASK(6,15)
-#define HTM_TRIGGER_CONTROL		5
+#define HTM_TRIGGER_CONTROL		(5 + dual_htm_offset)
 #define   HTM_CTRL_TRIG			PPC_BITMASK(0,1)
+#define   HTM_CTRL_MTSPR_TRIG		PPC_BIT(2)
+#define   HTM_CTRL_MTSPR_MARKER		PPC_BIT(3)
 #define   HTM_CTRL_MASK			PPC_BITMASK(4,5)
+#define   HTM_CTRL_DBG_TRIG0		PPC_BIT(6)
+#define   HTM_CTRL_DBG_TRIG1		PPC_BIT(7)
 #define   HTM_CTRL_XSTOP_STOP		PPC_BIT(13)
-#define NHTM_FILTER_CONTROL		6
+#define NHTM_FILTER_CONTROL		(6 + dual_htm_offset)
 #define   NHTM_FILTER_RCMD_SCOPE	PPC_BITMASK(17,19)
 #define   NHTM_FILTER_RCMD_SOURCE	PPC_BITMASK(20,21)
 #define   NHTM_FILTER_CRESP_PATTERN	PPC_BITMASK(27,31)
 #define   NHTM_FILTER_MASK		PPC_BITMASK(32,54)
 #define   NHTM_FILTER_CRESP_MASK	PPC_BITMASK(59,63)
-#define NHTM_TTYPE_FILTER_CONTROL	7
+#define NHTM_TTYPE_FILTER_CONTROL	(7 + dual_htm_offset)
 #define   NHTM_TTYPE_TYPE_MASK		PPC_BITMASK(17,23)
 #define   NHTM_TTYPE_SIZE_MASK		PPC_BITMASK(24,31)
-#define NHTM_CONFIGURATION		8
+#define NHTM_CONFIGURATION		(8 + dual_htm_offset)
 #define   NHTM_CONF_HANG_DIV_RATIO	PPC_BITMASK(0,4)
 #define   NHTM_CONF_RTY_DROP_COUNT	PPC_BITMASK(5,8)
 #define   NHTM_CONF_DROP_PRIORITY_INC	PPC_BIT(9)
 #define   NHTM_CONF_RETRY_BACKOFF	PPC_BIT(10)
 #define   NHTM_CONF_OPERALIONAL_HANG	PPC_BIT(11)
-#define NHTM_FLEX_MUX			9
+#define NHTM_FLEX_MUX			(9 + dual_htm_offset)
 #define   NHTM_FLEX_MUX_MASK		PPC_BITMASK(0,35)
 #define   NHTM_FLEX_DEFAULT		0xCB3456129
+#define   NHTM_FLEX_MUX_MASK_P10	PPC_BITMASK(0,39)
+#define   NHTM_FLEX_DEFAULT_P10		0xCB34561567
+#define NHTM_ADDR_PAT			0xb
+#define   NHTM_ADDR_PAT_MASK		PPC_BITMASK(0, 55)
+#define CHTM_PDBAR			0x0b
+#define   CHTM_PDBAR_PARITY		PPC_BIT(0)
+#define   CHTM_PDBAR_SCOPE		PPC_BITMASK(5, 7)
+#define   CHTM_PDBAR_ADDR		PPC_BITMASK(12, 52)
+#define NHTM_ADDR_MASK			0xc
+#define   NHTM_ADDR_MASK_MASK		PPC_BITMASK(0, 55)
+#define NHTM_STOP_FILTER		0xd
+#define   NHTM_STOP_FILTER_TTAG_PAT	PPC_BITMASK(0, 14)
+#define   NHTM_STOP_FILTER_TTAG_MASK	PPC_BITMASK(15, 29)
+#define   NHTM_STOP_FILTER_TTYPE_PAT	PPC_BITMASK(30, 36)
+#define   NHTM_STOP_FILTER_TTYPE_MASK	PPC_BITMASK(37, 43)
+#define   NHTM_STOP_FILTER_CRESP_PAT	PPC_BITMASK(44, 48)
+#define   NHTM_STOP_FILTER_CRESP_MASK	PPC_BITMASK(49, 53)
+#define   NHTM_STOP_CYCLES		PPC_BITMASK(54, 60)
+#define NHTM_STOP_FILT_ADDR_PAT		0xe
+#define   NHTM_STOP_FILT_ADDR_PAT_MASK	PPC_BITMASK(0, 55)
+#define NHTM_STOP_FILT_ADDR_MASK	0xf
+#define   NHTM_STOP_FILT_ADDR_MASK_MASK	PPC_BITMASK(0, 55)
+
+/* On P10 there are two nhtms which shifts some of the registers */
+static int dual_htm_offset;
 
 enum htm_state {
 	INIT,
@@ -161,11 +238,14 @@ enum htm_error {
 
 struct htm_status {
 	enum htm_state state;
+	enum htm_state state2;
 	enum htm_error error;
+	enum htm_error error2;
 	bool mem_size_select;
 	uint16_t mem_size;
 	uint64_t mem_base;
 	uint64_t mem_last;
+	uint64_t mem_last2;
 	uint64_t raw;
 };
 
@@ -292,6 +372,37 @@ static int get_status(struct htm *htm, struct htm_status *status)
 		status->error = NONE;
 	}
 
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm")) {
+		switch (status->raw & HTM2_STATUS_ERROR_MASK) {
+		case HTM2_STATUS_CRESP_OV:
+			status->error2 = CRESP_OV;
+			break;
+		case HTM2_STATUS_BUF_WAIT:
+			status->error2 = BUF_WAIT;
+			break;
+		case HTM2_STATUS_TRIG_DROPPED:
+			status->error2 = TRIGGER_DROPPED;
+			break;
+		case HTM2_STATUS_ADDR_ERROR:
+			status->error2 = ADDR_ERROR;
+			break;
+		case HTM2_STATUS_REC_DROPPED:
+			status->error2 = RECORD_DROPPED;
+			break;
+		case HTM2_STATUS_SCOM_ERROR:
+			status->error2 = SCOM_ERROR;
+			break;
+		case HTM2_STATUS_PARITY_ERROR:
+			status->error2 = PARITY_ERROR;
+			break;
+		case HTM2_STATUS_INVALID_CRESP:
+			status->error2 = INVALID_CRESP;
+			break;
+		default:
+			status->error2 = NONE;
+		}
+	}
+
 	switch (status->raw & HTM_STATUS_STATE_MASK) {
 	case HTM_STATUS_REPAIR:
 		status->state = REPAIR;
@@ -327,16 +438,66 @@ static int get_status(struct htm *htm, struct htm_status *status)
 		status->state = UNINITIALIZED;
 	}
 
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm")) {
+		switch (status->raw & HTM2_STATUS_STATE_MASK) {
+		case HTM2_STATUS_REPAIR:
+			status->state2 = REPAIR;
+			break;
+		case HTM2_STATUS_INIT:
+			status->state2 = INIT;
+			break;
+		case HTM2_STATUS_PREREQ:
+			status->state2 = PREREQ;
+			break;
+		case HTM2_STATUS_READY:
+			status->state2 = READY;
+			break;
+		case HTM2_STATUS_TRACING:
+			status->state2 = TRACING;
+			break;
+		case HTM2_STATUS_PAUSED:
+			status->state2 = PAUSED;
+			break;
+		case HTM2_STATUS_FLUSH:
+			status->state2 = FLUSH;
+			break;
+		case HTM2_STATUS_COMPLETE:
+			status->state2 = COMPLETE;
+			break;
+		case HTM2_STATUS_ENABLE:
+			status->state2 = ENABLE;
+			break;
+		case HTM2_STATUS_STAMP:
+			status->state2 = STAMP;
+			break;
+		default:
+			status->state2 = UNINITIALIZED;
+		}
+	}
+
 	if (HTM_ERR(pib_read(&htm->target, HTM_MEMORY_CONF, &val)))
 		return -1;
 
-	status->mem_size_select = val & HTM_MEM_SIZE_SMALL;
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm")) {
+		status->mem_size_select = val & HTM_MEM_SIZE_SMALL_P10;
+		status->mem_base = val & HTM_MEM_BASE_P10;
+	} else if (pdbg_target_compatible(&htm->target, "ibm,power10-chtm")) {
+		status->mem_size_select = val & HTM_MEM_SIZE_SMALL_P10;
+		status->mem_base = val & HTM_MEM_BASE_P10;
+	} else {
+		status->mem_size_select = val & HTM_MEM_SIZE_SMALL;
+		status->mem_base = val & HTM_MEM_BASE;
+	}
 	status->mem_size = GETFIELD(HTM_MEM_SIZE,val);
-	status->mem_base = val & HTM_MEM_BASE;
 
 	if (HTM_ERR(pib_read(&htm->target, HTM_LAST_ADDRESS, &status->mem_last)))
 		return -1;
 
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm")) {
+		if (HTM_ERR(pib_read(&htm->target, HTM2_LAST_ADDRESS, &status->mem_last2)))
+			return -1;
+	}
+
 	return 0;
 }
 
@@ -457,8 +618,7 @@ static int configure_debugfs_memtrace(struct htm *htm)
 
 static int configure_chtm(struct htm *htm, bool wrap)
 {
-	uint64_t hid0, ncu, val;
-	struct pdbg_target *core;
+	uint64_t val;
 
 	if (!pdbg_target_is_class(&htm->target, "chtm"))
 		return 0;
@@ -468,9 +628,37 @@ static int configure_chtm(struct htm *htm, bool wrap)
 
 	val = wrap ? HTM_MODE_WRAP : 0;
 	if (HTM_ERR(pib_write(&htm->target, HTM_COLLECTION_MODE,
-		HTM_MODE_ENABLE | val)))
+		HTM_MODE_ENABLE | HTM_MODE_DIS_TIMESTAMP |
+		HTM_MODE_CORE_TRACE_EN | val)))
 		return -1;
 
+	if (htm->configure && htm->configure(htm) < 0)
+		return -1;
+
+	return 0;
+}
+
+static int do_configure_chtm_p10(struct htm *htm)
+{
+	struct pdbg_target *core;
+	uint64_t mask;
+
+	/* P10 only: mask PC Logic checkstop in wrapping mode */
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-chtm")) {
+		core = pdbg_target_require_parent("core", &htm->target);
+		if (HTM_ERR(pib_read(core, CORE_FIR_MASK, &mask)))
+			return -1;
+		if (HTM_ERR(pib_write(core, CORE_FIR_MASK, mask | CORE_FIR_PC_LOGIC)))
+			return -1;
+	}
+	return 0;
+}
+
+static int do_configure_chtm_p8(struct htm *htm)
+{
+	uint64_t hid0, ncu;
+	struct pdbg_target *core;
+
 	core = pdbg_target_require_parent("core", &htm->target);
 	if (HTM_ERR(pib_read(core, HID0_REGISTER, &hid0)))
 		return -1;
@@ -489,12 +677,44 @@ static int configure_chtm(struct htm *htm, bool wrap)
 
 static int deconfigure_chtm(struct htm *htm)
 {
-	uint64_t hid0, ncu;
-	struct pdbg_target *core;
 
 	if (!pdbg_target_is_class(&htm->target, "chtm"))
 		return 0;
 
+	if (htm->deconfigure && htm->deconfigure(htm) < 0)
+		return -1;
+
+	if (HTM_ERR(pib_write(&htm->target, HTM_COLLECTION_MODE,0)))
+		return -1;
+
+//	FIXME this needs kernel work to happen
+//	if (HTM_ERR(deconfigure_debugfs_memtrace(htm)))
+//		return -1;
+	return 0;
+}
+
+static int do_deconfigure_chtm_p10(struct htm *htm)
+{
+	struct pdbg_target *core;
+	uint64_t mask;
+
+	core = pdbg_target_require_parent("core", &htm->target);
+	/* clear fir first before unmasking it */
+	if (HTM_ERR(pib_write(core, CORE_FIR_AND, ~CORE_FIR_PC_LOGIC)))
+		return -1;
+	if (HTM_ERR(pib_read(core, CORE_FIR_MASK, &mask)))
+		return -1;
+	if (HTM_ERR(pib_write(core, CORE_FIR_MASK, mask & ~CORE_FIR_PC_LOGIC)))
+		return -1;
+
+	return 0;
+}
+
+static int do_deconfigure_chtm_p8(struct htm *htm)
+{
+	struct pdbg_target *core;
+	uint64_t ncu, hid0;
+
 	core = pdbg_target_require_parent("core", &htm->target);
 	if (HTM_ERR(pib_read(core, NCU_MODE_REGISTER, &ncu)))
 		return -1;
@@ -508,12 +728,6 @@ static int deconfigure_chtm(struct htm *htm)
 	if (HTM_ERR(pib_write(core, HID0_REGISTER, hid0)))
 		return -1;
 
-	if (HTM_ERR(pib_write(&htm->target, HTM_COLLECTION_MODE,0)))
-		return -1;
-
-//	FIXME this needs kernel work to happen
-//	if (HTM_ERR(deconfigure_debugfs_memtrace(htm)))
-//		return -1;
 	return 0;
 }
 
@@ -566,6 +780,35 @@ static int configure_nhtm(struct htm *htm, bool wrap)
 		}
 	}
 
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm")) {
+		if (HTM_ERR(pib_read(&htm->target, NHTM_FLEX_MUX, &val)))
+			return -1;
+
+		if (GETFIELD(NHTM_FLEX_MUX_MASK_P10, val) != NHTM_FLEX_DEFAULT_P10) {
+			PR_ERROR("The HTM Flex wasn't default value\n");
+			return -1;
+		}
+
+		if (HTM_ERR(pib_write(&htm->target, NHTM_ADDR_PAT, 0)))
+			return -1;
+
+		if (HTM_ERR(pib_write(&htm->target, NHTM_ADDR_MASK, NHTM_ADDR_MASK_MASK)))
+			return -1;
+
+		if (HTM_ERR(pib_write(&htm->target, NHTM_STOP_FILTER,
+				      NHTM_STOP_FILTER_TTAG_MASK |
+				      NHTM_STOP_FILTER_TTYPE_MASK |
+				      NHTM_STOP_FILTER_CRESP_MASK)))
+			return -1;
+
+		if (HTM_ERR(pib_write(&htm->target, NHTM_STOP_FILT_ADDR_PAT, 0)))
+			return -1;
+
+		if (HTM_ERR(pib_write(&htm->target, NHTM_STOP_FILT_ADDR_MASK,
+				      NHTM_STOP_FILT_ADDR_MASK_MASK)))
+			return -1;
+	}
+
 	return 0;
 }
 
@@ -577,9 +820,13 @@ static int deconfigure_nhtm(struct htm *htm)
 	return 0;
 }
 
-static int is_startable(struct htm_status *status)
+static int is_startable(struct htm *htm, struct htm_status *status)
 {
-	return (status->state == READY || status->state == PAUSED);
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm"))
+		return ((status->state == READY || status->state == PAUSED) &&
+			(status->state2 == READY || status->state2 == PAUSED));
+	else
+		return (status->state == READY || status->state == PAUSED);
 }
 
 static char *get_debugfs_file(struct htm *htm, const char *file)
@@ -715,7 +962,12 @@ static int configure_memory(struct htm *htm)
 	small = 0;
 	if (size < 512*1024*1024)
 		small = 1;
-	val = SETFIELD(HTM_MEM_SIZE_SMALL, val, small);
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm"))
+		val = SETFIELD(HTM_MEM_SIZE_SMALL_P10, val, small);
+	else if (pdbg_target_compatible(&htm->target, "ibm,power10-chtm"))
+		val = SETFIELD(HTM_MEM_SIZE_SMALL_P10, val, small);
+	else
+		val = SETFIELD(HTM_MEM_SIZE_SMALL, val, small);
 	shift = 29; /* large */
 	if (small)
 		shift = 24;
@@ -728,7 +980,12 @@ static int configure_memory(struct htm *htm)
 	 * already shifted correct as read from the kernel, or'ing it
 	 * in works fine.
 	 */
-	val = SETFIELD(HTM_MEM_BASE, val, 0);
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm"))
+		val = SETFIELD(HTM_MEM_BASE_P10, val, 0);
+	else if (pdbg_target_compatible(&htm->target, "ibm,power10-chtm"))
+		val = SETFIELD(HTM_MEM_BASE_P10, val, 0);
+	else
+		val = SETFIELD(HTM_MEM_BASE, val, 0);
 	val |= base;
 
 	if (HTM_ERR(pib_write(&htm->target, HTM_MEMORY_CONF, val)))
@@ -761,7 +1018,7 @@ static int do_htm_reset(struct htm *htm, bool wrap)
 /* Stolen from p8chip.c */
 #define RAS_MODE_REG			0x1
 #define  MR_THREAD_IN_DEBUG		PPC_BIT(43)
-static int htm_toggle_debug_bit(struct htm *htm)
+static int do_post_configure_chtm_p8(struct htm *htm)
 {
 	struct pdbg_target *target;
 	struct pdbg_target *core;
@@ -773,12 +1030,6 @@ static int htm_toggle_debug_bit(struct htm *htm)
 	if (!core)
 		return 0; /* nhtm case */
 
-	/* FIXME: this is a hack for P8 */
-	if (!pdbg_target_compatible(core, "ibm,power8-core")) {
-		PR_ERROR("HTM is POWER8 only currently\n");
-		return -1;
-	}
-
 	pdbg_for_each_target("thread", core, target) {
 		if (pdbg_target_index(target) == 0) {
 			/* Need to set this bit to ensure HTM starts */
@@ -790,6 +1041,37 @@ static int htm_toggle_debug_bit(struct htm *htm)
 	return 0;
 }
 
+static int do_post_configure_chtm_p10(struct htm *htm)
+{
+	struct pdbg_target *core;
+	uint64_t hid0;
+
+	core = pdbg_target_require_parent("core", &htm->target);
+	if (HTM_ERR(pib_read(core, HIDQ_REGISTER, &hid0)))
+		return -1;
+	hid0 = HIDQ_ONE_PPC | HIDQ_DIS_PROC_REC | HIDQ_HILE;
+	if (HTM_ERR(pib_write(core, HIDQ_REGISTER, hid0)))
+		return -1;
+	hid0 |= HIDQ_EN_INST_TRACE;
+	if (HTM_ERR(pib_write(core, HIDQ_REGISTER, hid0)))
+		return -1;
+
+	return 0;
+}
+
+static int do_pre_deconfigure_chtm_p10(struct htm *htm)
+{
+	struct pdbg_target *core;
+	uint64_t hid0;
+
+	core = pdbg_target_require_parent("core", &htm->target);
+	hid0 = HIDQ_DIS_PROC_REC | HIDQ_HILE;
+	if (HTM_ERR(pib_write(core, HIDQ_REGISTER, hid0)))
+		return -1;
+
+	return 0;
+}
+
 static int __do_htm_start(struct htm *htm, bool wrap)
 {
 	struct htm_status status;
@@ -800,7 +1082,7 @@ static int __do_htm_start(struct htm *htm, bool wrap)
 	if (HTM_ERR(get_status(htm, &status)))
 		return -1;
 
-	if (!is_startable(&status)) {
+	if (!is_startable(htm, &status)) {
 		PR_ERROR("HTM not in a startable state!\n");
 		return -1;
 	}
@@ -812,7 +1094,14 @@ static int __do_htm_start(struct htm *htm, bool wrap)
 	if (HTM_ERR(pib_write(&htm->target, HTM_SCOM_TRIGGER, HTM_TRIG_START)))
 		return -1;
 
-	if (htm_toggle_debug_bit(htm))
+	while (1) {
+		if (HTM_ERR(get_status(htm, &status)))
+			return -1;
+		if (status.state == TRACING)
+			break;
+	}
+
+	if (htm->post_configure && htm->post_configure(htm))
 		return -1;
 
 	/*
@@ -833,6 +1122,7 @@ static int do_htm_start(struct htm *htm)
 static int do_htm_stop(struct htm *htm)
 {
 	struct htm_status status;
+
 	get_status(htm, &status);
 	if (HTM_ERR(get_status(htm, &status)))
 		return -1;
@@ -841,6 +1131,10 @@ static int do_htm_stop(struct htm *htm)
 		PR_INFO("* Skipping STOP trigger, HTM appears uninitialized\n");
 		return -1;
 	}
+
+	if (htm->pre_deconfigure && htm->pre_deconfigure(htm) < 0)
+		return -1;
+
 	if (status.state == TRACING) {
 		PR_INFO("* Sending STOP trigger to HTM\n");
 		if (HTM_ERR(pib_write(&htm->target, HTM_SCOM_TRIGGER, HTM_TRIG_STOP)))
@@ -1008,9 +1302,9 @@ static int do_htm_dump(struct htm *htm, char *filename)
 {
 	char *trace_file;
 	struct htm_status status;
-	uint64_t last, end, trace_size;
+	uint64_t last, end, trace_size, last2 = 0;
 	int trace_fd, dump_fd;
-	uint32_t eyecatcher;
+	uint64_t eyecatcher;
 	size_t r;
 	bool wrapped;
 
@@ -1025,6 +1319,11 @@ static int do_htm_dump(struct htm *htm, char *filename)
 		return -1;
 	}
 
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm") && status.state2 != COMPLETE) {
+		PR_INFO("* Skipping DUMP tigger, HTM2 is not in complete state\n");
+		return -1;
+	}
+
 	trace_file = get_debugfs_file(htm, "trace");
 	if (!trace_file)
 		return -1;
@@ -1036,7 +1335,13 @@ static int do_htm_dump(struct htm *htm, char *filename)
 		goto out3;
 	}
 
-	r = read(trace_fd, &eyecatcher, 4);
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm") ||
+	    pdbg_target_compatible(&htm->target, "ibm,power10-chtm")) {
+		lseek(trace_fd, 8, SEEK_SET);
+		r = read(trace_fd, &eyecatcher, 8);
+	} else {
+		r = read(trace_fd, &eyecatcher, 4);
+	}
 	if (r == -1) {
 		PR_ERROR("Failed to read from %s: %m\n", trace_file);
 		goto out2;
@@ -1045,6 +1350,8 @@ static int do_htm_dump(struct htm *htm, char *filename)
 	if (eyecatcher == 0x00f0efac)
 		wrapped = false;
 	last = status.mem_last - status.mem_base;
+	if (pdbg_target_compatible(&htm->target, "ibm,power10-nhtm"))
+		last2 = status.mem_last2 - status.mem_base;
 	end = htm_trace_size(&status);
 	trace_size = wrapped ? end : last;
 
@@ -1083,7 +1390,7 @@ static int do_htm_dump(struct htm *htm, char *filename)
 	r = lseek(trace_fd, 0, SEEK_SET);
 	if (r == -1)
 		goto out1;
-	r = copy_file(dump_fd, trace_fd, last);
+	r = copy_file(dump_fd, trace_fd, MAX(last2, last));
 	if (r)
 		goto out1;
 	r = 1;
@@ -1140,6 +1447,14 @@ static int nhtm_probe(struct pdbg_target *target)
 		}
 	}
 
+	switch (pdbg_get_proc()) {
+	case PDBG_PROC_P10:
+		dual_htm_offset = 0x01;
+		break;
+	default:
+		break;
+	}
+
 	return 0;
 }
 
@@ -1190,13 +1505,52 @@ static struct htm p8_chtm = {
 	.record = do_htm_record,
 	.status = do_htm_status,
 	.dump = do_htm_dump,
+	.configure = do_configure_chtm_p8,
+	.deconfigure = do_deconfigure_chtm_p8,
+	.post_configure = do_post_configure_chtm_p8,
 };
 DECLARE_HW_UNIT(p8_chtm);
 
+static struct htm p10_nhtm = {
+	.target = {
+		.name = "POWER10 Nest HTM",
+		.compatible = "ibm,power10-nhtm",
+		.class = "nhtm",
+		.probe = nhtm_probe,
+	},
+	.start = do_htm_start,
+	.stop = do_htm_stop,
+	.record = do_htm_record,
+	.status = do_htm_status,
+	.dump = do_htm_dump,
+};
+DECLARE_HW_UNIT(p10_nhtm);
+
+static struct htm p10_chtm = {
+	.target = {
+		.name = "POWER10 Core HTM",
+		.compatible = "ibm,power10-chtm",
+		.class = "chtm",
+		.probe = chtm_probe,
+	},
+	.start = do_htm_start,
+	.stop = do_htm_stop,
+	.record = do_htm_record,
+	.status = do_htm_status,
+	.dump = do_htm_dump,
+	.configure = do_configure_chtm_p10,
+	.deconfigure = do_deconfigure_chtm_p10,
+	.post_configure = do_post_configure_chtm_p10,
+	.pre_deconfigure = do_pre_deconfigure_chtm_p10,
+};
+DECLARE_HW_UNIT(p10_chtm);
+
 __attribute__((constructor))
 static void register_htm(void)
 {
 	pdbg_hwunit_register(PDBG_DEFAULT_BACKEND, &p8_nhtm_hw_unit);
 	pdbg_hwunit_register(PDBG_DEFAULT_BACKEND, &p9_nhtm_hw_unit);
+	pdbg_hwunit_register(PDBG_DEFAULT_BACKEND, &p10_nhtm_hw_unit);
 	pdbg_hwunit_register(PDBG_DEFAULT_BACKEND, &p8_chtm_hw_unit);
+	pdbg_hwunit_register(PDBG_DEFAULT_BACKEND, &p10_chtm_hw_unit);
 }
