@@ -9,6 +9,7 @@
 #define DATA (parse_number64, NULL)
 #define DATA32 (parse_number32, NULL)
 #define DATA16 (parse_number16, NULL)
+#define DATA8 (parse_number8, NULL)
 #define DEFAULT_DATA(default) (parse_number64, default)
 #define DEFAULT_DATA32(default) (parse_number32, default)
 #define GPR (parse_gpr, NULL)
@@ -17,6 +18,7 @@
 uint64_t *parse_number64(const char *argv);
 uint32_t *parse_number32(const char *argv);
 uint16_t *parse_number16(const char *argv);
+uint8_t *parse_number8(const char *argv);
 uint8_t *parse_number8_pow2(const char *argv);
 int *parse_gpr(const char *argv);
 int *parse_spr(const char *argv);
